#include <QApplication>
#include <iostream>

#include "mainWindow.h"
#include "mainCommandLine.h"

// Return gui or command line version of the application
// ======================================================================
QCoreApplication* createApplication(int &argc, char *argv[])
{
    for (int i = 1; i < argc; ++i)
        if (!qstrcmp(argv[i], "-gui"))
        {
            // Return a QT application which includes the GUI
            return new QApplication(argc, argv);
        }
    // Return a QT "core" application which does not contain a gui
    return new QCoreApplication(argc, argv);
}

// Main, either runs as command line or launches a gui (from cmd line)
// ===================================================================
int main(int argc, char* argv[])
{
    QScopedPointer<QCoreApplication> app(createApplication(argc, argv));

    // Show main window and launch QT event scheduler
    if (qobject_cast<QApplication *>(app.data()))
    {
        mainWindow mW;
        mW.show();
        return app->exec();
    }

    // else run in command line mode via mainCommandLine.run()
    else
    {
        QStringList inputArgs = app->arguments();
        mainCommandLine mainCL;
        mainCL.run(inputArgs);
    }
}
