#include <QDate>
#include <QTextStream>
#include <iostream>

#include "priceDataSet.h"

// Class to store / retrieve OHLC daily stock price data

priceDataSet::priceDataSet()
{
}

priceDataSet::~priceDataSet()
{
}

// Test / print all loaded data
void priceDataSet::test()
{
    // First check if we have some data
    if (getNumRows() > 1)
    {
        for (unsigned long i=0; i < getNumRows(); i++)
        {
            std::cout << getDate(i) << ","
                      << getOpen(i) << ","
                      << getHigh(i) << ","
                      << getLow(i) << ","
                      << getClose(i) << std::endl;
        }
    }
    else {
        std::cerr << "No data to print" << std::endl;
    }
}

// Convert string to unsigned in date
unsigned priceDataSet::getDateFromString(QString & inString)
{
    QDate date = QDate::fromString(inString,"yyyy-MM-dd");
    unsigned uDay = static_cast<unsigned>(date.day());
    unsigned uMonth = static_cast<unsigned>(date.month());
    unsigned uYear = static_cast<unsigned>(date.year());
    unsigned uDate = ((uYear*10000) + (uMonth*100) + (uDay));
    return uDate;
}


// Read a csv OHLC file from an open QT device
bool priceDataSet::Read(QIODevice *device)
{

    enum LINE_PARSE_INDEX
    {
        DATE = 0,
        OPEN = 1,
        HIGH = 2,
        LOW = 3,
        CLOSE = 4,
    };

    // Clear the gui
    int lineNum=0;

    // Create a thread to retrieve data from a file
    QTextStream in(device);

    //Read data up to the end of file
    while (!in.atEnd())
    {
        // Read in lines from the file
        QString line = in.readLine();
        if (lineNum > 1)
        {
            int dataIndex=0;
            unsigned date;
            double open,high,low,close,vol;

            date=0;
            open=high=low=close=vol=0.0;

            std::vector<double> priceData;

            for (QString datum : line.split(",")) {
                // First part of the line is the date
                if (dataIndex == DATE)
                {
                    date = getDateFromString(datum);
                }
                else if (dataIndex == OPEN)
                {
                    open = datum.toDouble();
                }
                else if (dataIndex == HIGH)
                {
                    high = datum.toDouble();
                }
                else if (dataIndex == LOW)
                {
                    low = datum.toDouble();
                }
                else if (dataIndex == CLOSE)
                {
                    close = datum.toDouble();
                }
                else
                {
                    // Do nothing, we're not using adj close or volume
                }
                dataIndex++;
            }

            // Create priceDatum element
            priceDatum newDataPoint = priceDatum(date,open,high,low,close);

            // Push element on to data vector
            priceData_.push_back(newDataPoint);
        }
        lineNum++;
    }

    device->close();
    return true;
}

// Get number of rows (days)
unsigned long priceDataSet::getNumRows()
{
    return priceData_.size();
}

// Get the date
unsigned priceDataSet::getDate(unsigned long row)
{
    return priceData_.at(row).getDate();
}

// Get the open
double priceDataSet::getOpen(unsigned long row)
{
    return priceData_.at(row).getOpen();
}

// Get the high
double priceDataSet::getHigh(unsigned long row)
{
    return priceData_.at(row).getHigh();
}

// Get the low
double priceDataSet::getLow(unsigned long row)
{
    return priceData_.at(row).getLow();
}

// Get the close
double priceDataSet::getClose(unsigned long row)
{
    return priceData_.at(row).getClose();
}
