#include "simpleMovingAverage.h"

// Semi-efficient quick moving average class

// Constructor
simpleMovingAverage::simpleMovingAverage(unsigned iLen)
{
    mAvgLength_ = iLen;
    if (iLen < 1)
    {
        iLen = 1;
    }
    dataPoints_ = std::unique_ptr<boost::circular_buffer<double>>(new boost::circular_buffer<double>(iLen));
    numDataPointsRead_ = 0;
    currentValue_ = 0.0;
}

// Add data points (double) and (int)
void simpleMovingAverage::addDataPoint(double iVal)
{
    // We haven't read mAvgLength_ worth of data yet, calculate using "slow" method
    if (spoolSamples())
    {
        dataPoints_->push_back(iVal);
        numDataPointsRead_++;
        if (calcSimple())
        {
            double sum = 0.0;
            for (unsigned i=0;i<mAvgLength_;i++)
            {
                double dataPoint = dataPoints_->at(i);
                sum += dataPoint;
            }
            currentValue_ = sum/mAvgLength_;
        }
    }

    // We have read mAvgLength_ worth of data, calculate via quick method
    else if (calcQuick())
    {
        // Calculate sum for current avg length
        double avgSum = currentValue_*mAvgLength_;
        // Subtract nth ago (0) element
        double nthElement = dataPoints_->at(0);
        avgSum -= nthElement;
        // Add latest value
        avgSum += iVal;
        // Re calculate the average
        currentValue_ = avgSum / mAvgLength_;
        // Add current value
        dataPoints_->push_back(iVal);
        numDataPointsRead_++;
    }
}

void simpleMovingAverage::addDataPoint(int iVal)
{
    double dVal = static_cast<double>(iVal);
    addDataPoint(dVal);
}


// Logic for calculating the average
bool simpleMovingAverage::calcSimple()
{
    return (numDataPointsRead_ == mAvgLength_);
}

bool simpleMovingAverage::calcQuick()
{
    return (numDataPointsRead_ >= mAvgLength_);
}

bool simpleMovingAverage::spoolSamples()
{
    return (numDataPointsRead_ < mAvgLength_);
}

// Return current value
double simpleMovingAverage::getCurrentValue()
{
    return currentValue_;
}
