#include "stockPortfolio.h"

// Portfolio class to help manage a stock portfolio

// Constructor
stockPortfolio::stockPortfolio(double start)
{
    cashAmount_ = start;
    startAmount_ = start;
    numShares_ = 0;
    numSells_ = 0;
    numTradesPos_ = 0;
    numTradesNeg_ = 0;
    minValue_ = start;
    maxValue_ = start;
}

// Accessors
double stockPortfolio::getCashValue()
{
    return cashAmount_;
}

// Get the number of shares held in the portfolio
unsigned stockPortfolio::getNumShares()
{
    return numShares_;
}

// Get the invested value
double stockPortfolio::getInvestedValue()
{
    return investedValue_;
}

// Get the total value
double stockPortfolio::getTotalValue()
{
    return totalValue_;
}

// Get portfolio start value
double stockPortfolio::getStartValue()
{
    return startAmount_;
}

// Get minimum total value
double stockPortfolio::getMinValue()
{
    return minValue_;
}

// Get maximum total value
double stockPortfolio::getMaxValue()
{
    return maxValue_;
}

// Get total number of trades (buys + sells)
unsigned stockPortfolio::getNumSellTrades()
{
    return numSells_;
}

// Get the number of profitable trades
unsigned stockPortfolio::getNumTradesPos()
{
    return numTradesPos_;
}

// Get the number of trades that incurred a loss
unsigned stockPortfolio::getNumTradesNeg()
{
    return numTradesNeg_;
}

// Update portfolio value and some metrics
void stockPortfolio::update(double inPrice)
{
    investedValue_ = numShares_*inPrice;
    totalValue_ = cashAmount_ + investedValue_;
    if (totalValue_ > maxValue_)
    {
        maxValue_ = totalValue_;
    }
    else if (totalValue_ < minValue_)
    {
        minValue_ = totalValue_;
    }

}

// BUY inNumShares shares at inBuyPrice
void stockPortfolio::doBuy(double inBuyPrice, unsigned inNumShares)
{
    double amountToSpend = inBuyPrice*inNumShares;
    double currentCash = cashAmount_;

    // Adjust cash
    currentCash -= amountToSpend;
    cashAmount_ = currentCash;
    // Num shares
    unsigned currentShares = numShares_;
    currentShares += inNumShares;
    numShares_ = currentShares;

    buyPrice_ = inBuyPrice;
}

// Sell all currently owned shares at inSalePrice
void stockPortfolio::doSell(double inSalePrice)
{
    double currentCash = cashAmount_;
    double totalSale = inSalePrice * numShares_;
    double newPortVal = currentCash+totalSale;
    cashAmount_ = newPortVal;
    numShares_ = 0;

    numSells_++;
    sellPrice_ = inSalePrice;
    if (sellPrice_ > buyPrice_)
    {
        numTradesPos_++;
    }
    else if (sellPrice_ < buyPrice_)
    {
        numTradesNeg_++;
    }

}
